
package planet.behaviour;

import planet.commonapi.Node;
import planet.commonapi.RouteMessage;

/**
* This interface provides a method to add functionality
* to a behvaiour.  
* @author Marc Sanchez
*/
public interface Behaviour {
	/**
	 *  Given a RouteMessage and a Node as input, onMessage's method do execution 
	 * steps for the behaviour. Node should be casted to Node's class in order to
	 * manage data structures and protocol of designed overlay.
	 * @param msg RouteMessage taken as input.
	 * @param node Node taken as input.
	 * @return Returns either an array of RouteMessages or null when no messages
	 * need to transmit this node. 
	 */
	public RouteMessage[] onMessage(RouteMessage msg, Node node);
	/**
	 * @return Returns the name of behaviour. 
	 */
	public String getName();
	/**
	 * @return Returns a string representation of the behaviour. In general, the toString
	 * method returns a string that "textually represents" this behaviour. The result should 
	 * be a concise but informative representation that is easy for a person to read. 
	 */ 
	public String toString();
}
