package planet.commonapi.GML;

import planet.commonapi.Id;
import planet.commonapi.exception.InitializationException;

/**
 * Its objective is build any required element to build an GML graph.
 * @author <a href="mailto: jordi.pujol@estudiants.urv.es">Jordi Pujol</a>
 * 21/02/2005
 */
public interface GMLFactory extends java.io.Serializable{

    /**
     * Build a GML edge with the specified values. The fill color is used the predefined one
     * (the appeared in GMLDefaultProperties.fill property).
     * @param source Source node Id.
     * @param target Destination node Id.
     * @param directed true when the edge is directed. false in other case.
     * @return An instance of GMLEdge.
     * @throws InitializationException if an error occurs during the initialization.
     */
    public GMLEdge buildEdge(Id source, Id target, boolean directed) throws InitializationException;
    
    /**
     * Build a GML edge with the specified values. The fill color is used the predefined one
     * (the appeared in GMLDefaultProperties.fill property).
     * @param source Source node Id.
     * @param target Destination node Id.
     * @param directed true when the edge is directed. false in other case.
     * @param fill Fill color in "#RRGGBB" format.
     * @return An instance of GMLEdge.
     * @throws InitializationException if an error occurs during the initialization.
     */
    public GMLEdge buildEdge(Id source, Id target, boolean directed, String fill) throws InitializationException;
    
    /**
     * Builds the GMLGenerator specified in the GML properties file.
     * @return The GMLGenerator.
     * @throws InitializationException if an error occurs during the initialization.
     */
    public GMLGenerator buildGMLGenerator() throws InitializationException ;
    
    /**
     * Build the GMLConstraint specified in the GML properties file.
     * @return The GMLConstraint.
     * @throws InitializationException if an error occurs during the initialization.
     */
    public GMLConstraint buildDefaultGMLConstraint() throws InitializationException ;
    
}
