package planet.generic.commonapi;

import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.factory.GenericFactory;
import planet.util.Properties;

/**
 * This class only attemps to help to programmers make more simple the initialization of the 
 * required context of new applications. Are two ways to initialize the context:
 * <br><br><b>One</b><br>
 * The new application must extends this GenericApp and includes in theirs constructors the
 * sentence <b>super()</b> or <b>super(factoriesProperties,simProperties,kbrProperties)</b>.
 * For example:
 * <b><pre>
 * public NewTest() throws InitializationException {
 *     super(factoriesProperties,simProperties,kbrProperties);
 * </pre></b>
 * where parameters of super() constructor are the path to specified files.
 * <br><br><b>Two</b><br>
 * Includes theese lines before anything that make reference to the network and the simulator and ...
 * <b><pre>
 * Properties.init(factoriesProperties,simProperties,kbrProperties);
 * GenericFactory.init();
 * </pre></b>
 * where parameters of init() method are the path to specified files.
 * 
 * @author Jordi Pujol
 */
public class GenericApp {
	
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");
  	 * GenericFactory.init();
  	 * </pre></b>
  	 * Because of theese lines, the new application must be executed under <b>bin</b> directory.
	 * 
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp() throws InitializationException {
  		Properties.init("../conf/factories.properties","../conf/sim.properties","../conf/chord.properties");
  		GenericFactory.init();
	}
	
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init(factoriesProperties,simProperties,kbrProperties);
     * GenericFactory.init();
	 * </pre></b>
	 * Because of theese lines, the new application must be executed under the implicitly specified
	 * directory by the parameters files.
	 * @param factoriesProperties Path to properties file of factories attributes.
	 * @param simProperties Path to properties file of simulator attributes.
	 * @param kbrProperties Path to properties file of chord attributes.
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp(String factoriesProperties,String simProperties, String kbrProperties) throws InitializationException {
  		Properties.init(factoriesProperties,simProperties,kbrProperties);
  		GenericFactory.init();
	}
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init(factoriesProperties,simProperties,chordProperties);
     * GenericFactory.init();
	 * </pre></b>
	 * Because of theese lines, the new application must be executed under the implicitly specified
	 * directory by the parameters files.
	 * @param factoriesProperties Path to properties file of factories attributes.
	 * @param simProperties Path to properties file of simulator attributes.
	 * @param kbrProperties Path to properties file of kbr attributes.
	 * @param dhtProperties Path to properties file of dht attributes.
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp(String factoriesProperties,String simProperties, String kbrProperties, String dhtProperties) throws InitializationException {
  		Properties.init(factoriesProperties,simProperties,kbrProperties,dhtProperties);
  		GenericFactory.init();
	}
	/**
	 * Make required initialization on the context of the application. In concrete:
	 * <b><pre>
	 * Properties.init(factoriesProperties,simProperties,chordProperties);
     * GenericFactory.init();
	 * </pre></b>
	 * Because of theese lines, the new application must be executed under the implicitly specified
	 * directory by the parameters files.
	 * @param factoriesProperties Path to properties file of factories attributes.
	 * @param simProperties Path to properties file of simulator attributes.
	 * @param kbrProperties Path to properties file of kbr attributes.
	 * @param dhtProperties Path to properties file of dht attributes.
	 * @param behProperties Paht to properties file of behaviours attributes
	 * @throws InitializationException if any error has occurred during initialization of the context.
	 */
	public GenericApp(String factoriesProperties,String simProperties, String kbrProperties, String dhtProperties, String behProperties) throws InitializationException {
  		Properties.init(factoriesProperties,simProperties,kbrProperties,dhtProperties, behProperties);
  		GenericFactory.init();
	}
	
	/**
	 * Shows in Y.XXX string format, the seconds elapsed between <b>firstTime</b> and </b>lastTime</b>,
	 * where 'Y' are the seconds, and 'XXX' the milliseconds ones.
	 * The time elapsed is obtained as <b>lastTime-firstTime</b>.
	 * @param firstTime First logged time.
	 * @param lastTime Second logged time.
	 * @return The seconds elapsed between the two logged times, in Y.XXX format.
	 */
	public static String timeElapsedInSeconds(long firstTime, long lastTime)
	{
	    long timeElapsed = lastTime-firstTime;
	    return 
	    		(timeElapsed/1000)+"."+  //the seconds
	    			((timeElapsed<10)?"00"+timeElapsed:  //up to 9 milliseconds
	        		 (timeElapsed<100)?"0"+timeElapsed:  //up to 99 milliseconds
	        		 ""+(timeElapsed%1000));             //up to 999 millisecons
	}
}
