package planet.simulate;

import java.lang.reflect.Constructor;
import java.util.Stack;

import planet.commonapi.*;
import planet.commonapi.exception.InitializationException;
import planet.commonapi.RouteMessage;
import planet.util.Properties;

/**
 * @author Pedro Garca
 * @author Carles Pairot
 */
public class MessagePool {
  /**
   * RouteMessages just created.
   */
  private static Stack messages           = null;
  /**
   * Only for temporal use.
   */
  private static RouteMessage tmp         = null;
  /**
   * Constructor to generate new instances of RouteMessages.
   */
  private static Constructor routeMessage = null; //is initialized the first time

  public static void init () throws InitializationException {
  	messages = new Stack();
  	try {
  		//String appId, NodeHandle to, NodeHandle from, NodeHandle nh, 
		//planet.commonapi.Message msg, String key, int type, int mode
      	Class[] types = {String.class,NodeHandle.class,NodeHandle.class,NodeHandle.class,planet.commonapi.Message.class,String.class,int.class,int.class};
		routeMessage = Properties.routeMessage.getConstructor(types);
  	} catch (Exception e) {
  		throw new InitializationException("Initialization failed.",e);
  	}
  }
  /**
   * Return a RouteMessage with the specified values. If there are RouteMessages free,
   * just build a new one. For generate new instances of RouteMessage will be used
   * the implementation class that appears in properties file. For accessing it, this
   * method use planet.util.Properties.routeMessage attribute.
   * @param key Identification of communication.
   * @param from Source node.
   * @param to Destination node.
   * @param type Type of message.
   * @param mode Mode of message. 
   * @return A RouteMessage with the specified values.
   */ //key,from,to,type,mode
  public static RouteMessage getMessage(String key,NodeHandle from, NodeHandle to,int type, int mode) throws InitializationException {
    if (messages.size() > 0) {
      tmp = (RouteMessage)messages.pop();
      //String appId,Id to,Id from,
		//Id nextHop, Message msg,String key,int type, int mode
      //tmp.setValues (appId,to,from,nextHop,null,"",Globals.BROADCAST,Globals.REFRESH);
      tmp.setValues ("",to,from,to,null,key,type,mode);
      return (RouteMessage)tmp;
    } else {
      Object[] params = {"",to,from,to,null,key,new Integer(type),new Integer(mode)};
      try {
      	return (RouteMessage)routeMessage.newInstance(params); 
      } catch (Exception e) {
      	throw new InitializationException ("Cannot generate a new instance of message.",e);
      }
      /*
       * Builds a new instance of RouteMessage with all specified values.
       * @param from NodeHandle who sends this message.
       * @param to NodeHandle that identifies the last receiver.
       */

    }
  }
  

  /**
   * Builds a new instance of RouteMessage with all specified values.
   * @param key Key of the message.
   * @param from NodeHandle who sends this message.
   * @param to NodeHandle that identifies the last receiver.
   * @param nextHop NodeHandle that identifies the next hop.
   * @param msg Message to be send in this RouteMessage.
   * @param type Type of the message.
   * @param mode Mode of the message.
   * @param appId Application that has build this <b>msg</b>.
   * @return A RouteMessage with all specified values.
   * @throws InitializationException if cannot build a new instance
   * of the RouteMessage.
   */
  public static RouteMessage getMessage(String key, 
  		NodeHandle from, NodeHandle to, NodeHandle nextHop, Message msg, 
		int type, int mode, String appId) throws InitializationException {
    if (messages.size() > 0) {
      tmp = (RouteMessage)messages.pop();
      //String appId,Id to,Id from,
		//Id nextHop, Message msg,String key,int type, int mode
      tmp.setValues (appId,to,from,nextHop,msg,key,type,mode);
      return tmp;
    } else {
      Object[] params = {appId,to,from,nextHop,msg,key,new Integer(type),new Integer(mode)};
      try {
      	return (RouteMessage)routeMessage.newInstance(params); 
      } catch (Exception e) {
      	throw new InitializationException ("Cannot generate a new instance of RouteMessage.",e);
      }
    }
  }
  
  /**
   * To free a message only pushing it to stack to reuse.
   * @param aMsg A RouteMessage to be released.
   */
  public static void freeMessage (planet.commonapi.RouteMessage aMsg) {
    messages.push (aMsg);
  }
}
