package planet.symphony.behaviours;

import planet.behaviour.Behaviour;
import planet.commonapi.EndPoint;
import planet.commonapi.Id;
import planet.commonapi.Node;
import planet.commonapi.NodeHandle;
import planet.commonapi.RouteMessage;
import planet.simulate.Results;

/**
 * DataBehaviour handles DATA request messages. When a node receives a DATA request addressed to
 * it, then it forwads it to application layer through the commonapi interface.
 * 
 * @author Marc Sanchez <marc.sanchez@estudiants.urv.es>
 * @author Helio Tejedor <heliodoro.tejedor@estudiants.urv.es>
 */
public class DataBehaviour implements Behaviour {

	/**
	 *  Given a RouteMessage and a Node as input, onMessage's method do execution 
	 * steps for the behaviour. Node should be casted to Node's class in order to
	 * manage data structures and protocol of designed overlay.
	 * @param msg RouteMessage taken as input.
	 * @param node Node taken as input.
	 * @return Returns either an array of RouteMessages or null when no messages
	 * need to transmit this node. 
	 */
	public RouteMessage[] onMessage(RouteMessage msg, Node node) {
	    EndPoint endpoint = node.getRegisteredApplication(msg.getApplicationId());
	    Id pred = node.getPred().getId();
	    Id myId = node.getId();
	    NodeHandle bestEffort = node.getClosestNodeHandle(msg.getDestination().getId());
	    
	    if (myId.equals(msg.getDestination().getId()) ||
	        msg.getDestination().getId().betweenE(pred,myId) ||
	        myId.equals(bestEffort.getId()))
	    {
	        //Deliver message to End Point
		    boolean forward = endpoint.forward(msg);
		    if (forward) {
		        endpoint.scheduleMessage(msg, 0);
		    } 
		    Results.decTraffic();
		    return null;
		}
	    
	    msg.setNextHopHandle(bestEffort);
	    //endpoint.forward(msg);
	    if (endpoint.forward(msg))
	        return new RouteMessage[]{msg};
	    return null;
	}
	
	/**
	 * @return Returns the name of behaviour. 
	 */
	public String getName() {
		return "DataBehaviour";
	}
	/**
	 * @return Returns a string representation of the behaviour. In general, the toString
	 * method returns a string that "textually represents" this behaviour. The result should 
	 * be a concise but informative representation that is easy for a person to read. 
	 */ 
	public String toString() {
		return getName();
	}
}
