package planet.util;

import java.io.*;

import planet.commonapi.exception.InitializationException;
import planet.generic.commonapi.factory.Topology;

/**
 * 
 * @author Jordi Pujol
 * @author Marc Sanchez
 */
public class Properties {
	
	/* ************************** PROPERTIES FILES *************************/
	/**
	 * Default file which contains the default properties for the factories.
	 */
	public static final String DEFAULT_FACTORY_PROPERTIES = "../conf/factories.properties";
	/**
	 * Default file which contains the default properties for the simulator.
	 */
	public static final String DEFAULT_SIM_PROPERTIES = "../conf/sim.properties";
	/**
	 * Default file which contains the default properties for the Chord protocol.
	 */
	public static final String DEFAULT_KBR_PROPERTIES = "../conf/chord.properties";
	/**
	 * Default file which contains the default properties for the DHT application layer.
	 */
	public static final String DEFAULT_DHT_APP_PROPERTIES = "../conf/DHT.properties";
	
	/* ************************** PROPERTIES FOR FACTORIES ********************/
	/**
	 * Factories property: The default key specified in the properties file that
	 * identifies the class of the network implementation to use.
	 */
	public static final String DEFAULT_NETWORK = "DEFAULT_NETWORK";
	
	/**
	 * Factories property: The default key specified in the properties file that identifies
	 * the class of some Application's implementation.
	 */
	public static final String DEFAULT_APPLICATION = "DEFAULT_APPLICATION";
	/**
	 * Factories property: The default Class for Ids specified in the properties file.
	 */
	public static final String DEFAULT_ID_CLASS = "DEFAULT_IDCLASS";
	/**
	 * Factories property: The default IdFactory key specified in the properties file.
	 */
	public static final String DEFAULT_IDFACTORY = "DEFAULT_IDFACTORY";
	/**
	 * Factories property: The default Class for BehavioursPools specified in the properteis file.
	 */
	public static final String DEFAULT_BEHCLASS = "DEFAULT_BEHCLASS"; 
	/**
	 * Factories property: The default BehavioursPoolFactory key specified in the properties file.
	 */
	public static final String DEFAULT_BEHFACTORY = "DEFAULT_BEHFACTORY";
	/**
	 * Factories property: The default Class for Nodes specified in the properties file.
	 */
	public static final String DEFAULT_NODECLASS = "DEFAULT_NODECLASS";
	/**
	 * Factories property: The default Class for NodeHandles specified in the properties file.
	 */
	public static final String DEFAULT_NODEHANDLECLASS = "DEFAULT_NODEHANDLECLASS";
	/**
	 * Factories property: Default key at properties file that identifies the NodeFactory.
	 */
	public static final String DEFAULT_NODE_FACTORY    = "DEFAULT_NODEFACTORY";
	/**
	 * Factories property: Default key at properties file that identifies the network size.
	 */
	public static final String DEFAULT_NETWORK_SIZE    = "DEFAULT_SIZE";
	/**
	 * Factories property: Default key at properties file that identifies the number of bits of the key.
	 */
	public static final String DEFAULT_BITS_KEY        = "DEFAULT_K";
	/**
	 * Factories property: Default key at properties file that identifies the serialized file that
	 * contains the state to load.
	 */
	public static final String DEFAULT_SERIALIZED_FILE = "DEFAULT_SERIALIZEDFILE";
	/**
	 * Factories property: Default key at properties file that identifies the topology of the network.
	 */
	public static final String DEFAULT_TOPOLOGY = "DEFAULT_TOPOLOGY";
	/**
	 * Factories property: Default key at properties file that identifies the class for new RouteMessage. 
	 */
	public static final String DEFAULT_ROUTE_MESSAGE = "DEFAULT_ROUTEMESSAGE";
	/**
	 * Factories property: Default key at properties file that identifies the class for new EndPoints.
	 */
	public static final String DEFAULT_END_POINT = "DEFAULT_ENDPOINT";
	
	/* *********************** SIMULATOR PROPERTIES ***********************/
	/**
	 * Simulator property: Default key for filename with events to load.
	 */
	public static final String DEFAULT_EVENT_FILE = "DEFAULT_EVENT_FILE";
	/**
	 * Simulator property: Default key for number of stabilization steps for any node join or leave.
	 */
	public static final String DEFAULT_SIMULATION_STEPS = "DEFAULT_SIMULATION_STEPS";
	/**
	 * Simulator property: Default key for log level.
	 */
	public static final String DEFAULT_LOG_LEVEL = "DEFAULT_LOG_LEVEL";
	/**
	 * Simulator property: Default key for filename to which serialize the final state.
	 */
	public static final String DEFAULT_OUTPUT_FILE = "DEFAULT_OUTPUT_FILE";
	/**
	 * Simulator property: Default key to show if the output file must be replaced with new outputs.
	 */
	public static final String DEFAULT_REPLACE_OUTPUT_FILE = "DEFAULT_REPLACE_OUTPUT_FILE";
	/**
	 * Simulator property: Default key for Timer class to use at nodes.
	 */
	public static final String DEFAULT_TIMER = "DEFAULT_TIMER";
	/**
	 * Simulator property: Default key for the overlay properties class.
	 */
	public static final String DEFAULT_OVERLAY_PROPERTIES = "DEFAULT_OVERLAY_PROPERTIES";
	/* *********************** OVERLAY PROPERTIES ***********************/
	/**
	 * Kbr property: Default key for queue size.
	 */
	public static final String DEFAULT_QUEUE_SIZE = "DEFAULT_QUEUE_SIZE";
	/**
	 * Kbr property: Default key for message processig.
	 */
	public static final String DEFAULT_MSG_PROCESS = "DEFAULT_MSG_PROCESS";
	/**
	 * Simulator property: Default key for the DHT properties class.
	 */
	public static final String DEFAULT_DHT_PROPERTIES = "DEFAULT_DHT_PROPERTIES";
	/**
	 * Simulator property: Default key for the Behaviour properties class.
	 */
	public static final String DEFAULT_BEHAVIOUR_PROPERTIES = "DEFAULT_BEHAVIOUR_PROPERTIES";
	/**
	 * Simulator property: Default key for the graphical GML properties class.
	 */
	public static final String DEFAULT_GML_PROPERTIES = "DEFAULT_GML_PROPERTIES";
	
	/* *********************** DHT PROPERTIES ***********************/
	/**
	 * This contains all properties for the concrete dht.
	 */
	public static DHTProperties dhtProps;
	/**
	 * Class with all dht properties.
	 */
	public static Class dhtPropertiesClass;
	/* ****************** BEHAVIOUR PROPERTIES *********************/
	/**
	 * This contains all patterns for all behaviours.
	 */
	public static BehaviourProperties behProps;
	/**
	 * Class with all behaviours
	 */
	public static Class behPropertiesClass;
	
	/* *************** ATTRIBUTES FOR FACTORIES  ****************/
	/**
	 * Class of Network implementation.
	 */
	public static Class networkClass;
	/**
	 * Class of Application implementation.
	 */
	public static Class appClass;
	/**
	 * Topology of the target network.
	 */
	public static String topology;
	/**
	 * Id class.
	 */
	public static Class idClass;
	/**
	 * BehavioursPool's class.
	 */
	public static Class behClass;
	/**
	 * Network size to build.
	 */
	public static int networkSize;
	/**
	 * NodeFactory implementation class.
	 */
	public static Class nodeFactClass;
	/**
	 * IdFactory implementation class.
	 */
	public static Class idFactory;
	/**
	 * BehavioursPool's Factory class.
	 */
	public static Class behFactoryClass; 
	/**
	 * Node implementation class.
	 */
	public static Class nodeClass;
	/**
	 * NodeHandle implementation class.
	 */
	public static Class nodeHandleClass;
	/**
	 * Number of bits for Id.
	 */
	public static int bitsKey;
	/**
	 * Filename to use to load some serialized state.
	 */
	public static String serializedFile;
	/**
	 * Class that implements RouteMessage interface.
	 */
	public static Class routeMessage;
	/**
	 * Implementation of  EndPoint interface.
	 */
	public static Class endPoint;
	
	/* *************** ATTRIBUTES FOR SIMULATOR  ****************/
	/**
	 * Filename to use to load events to the simulator.
	 */
	public static String eventFile;
	/**
	 * Number of steps to be runned after any node join or leave.
	 */
	public static int simulationSteps;
	/**
	 * Log level during simulation..
	 */
	public static int logLevel;
	/**
	 * Filename to use to save the actual state by serialization.
	 */
	public static String outputFile; 
	/**
	 * Flag to show if the output file must be replaced or not.
	 */
	public static boolean replaceOutputFile;
	/**
	 * Class to use for Timers at nodes.
	 */
	public static Class timerClass;
	/**
	 * Class with all overlay properties.
	 */
	public static Class overlayPropertiesClass;
	/**
	 * Size of message queues.
	 */
	public static int queueSize;
	/**
	 * Message processing.
	 */
	public static int msgProcess;

	/* *************** ATTRIBUTES FOR OVERLAY  ****************/
	/**
	 * This contains all properties for the concrete overlay.
	 */
	public static OverlayProperties overlayProps;
	
	
	/* *************** ATTRIBUTES FOR GML *********************/
	
	/** Have all graphical properties for GML purposes. */
	public static PropertiesInitializer gmlProps;
	/** Implementation class for the graphical attributes on GML graphs. */
	public static Class gmlPropsClass;
	/** Usable instance of the GMLFactory in use. */
	public static String gmlClassesProperties;
	
	/**
	 * Load all properties for factories, simulator and chord, from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void init(String factoryProperties,String simProperties,String overlayProperties)
			throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);		
	}
	/**
	 * Load all properties for factories, simulator, overlay, DHT from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
     * @param behProperties File that contains all behaviours
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void init(String factoryProperties,String simProperties,String overlayProperties,
	        String behProperties) throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);	
		initBehaviour(behProperties);
	}
	/**
	 * Load all properties for factories, simulator, overlay, DHT from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
	 * @param dhtProperties File that contains the DHT properties.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void init(String factoryProperties,String simProperties,String overlayProperties, 
	        String behProperties, String dhtProperties)	throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);
		initBehaviour(behProperties);
		initDHT(dhtProperties);
	}
	
	/**
	 * Load all properties for factories, simulator and chord, from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
	 * @param gmlProperties  File that contains the graphical GML properties.
	 * @param gmlClassesProperties File that contains the implementations classes for GML graphs.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void initWithGML(String factoryProperties,String simProperties,String overlayProperties,
	        String gmlProperties, String gmlClassesProperties) throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);
		initGraphicalGML(gmlProperties);
		initGMLClasses(gmlClassesProperties);
	}
	/**
	 * Load all properties for factories, simulator, overlay, DHT from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
     * @param behProperties File that contains all behaviours
	 * @param gmlProperties  File that contains the graphical GML properties.
	 * @param gmlClassesProperties File that contains the implementations classes for GML graphs.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void initWithGML(String factoryProperties,String simProperties,String overlayProperties,
	        String behProperties, String gmlProperties,String gmlClassesProperties) throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);	
		initBehaviour(behProperties);
		initGraphicalGML(gmlProperties);
		initGMLClasses(gmlClassesProperties);
	}
	/**
	 * Load all properties for factories, simulator, overlay, DHT from the
	 * specified files, respectively.
	 * @param factoryProperties File that contains the factories properties.
	 * @param simProperties File that contains the simulator properties. 
	 * @param overlayProperties  File that contains the overlay properties.
	 * @param dhtProperties File that contains the DHT properties.
 	 * @param gmlProperties  File that contains the graphical GML properties.
	 * @param gmlClassesProperties File that contains the implementations classes for GML graphs.
	 * @throws InitializationException if some error occur during initialization.
	 */
	public static void initWithGML(String factoryProperties,String simProperties,String overlayProperties, 
	        String behProperties, String dhtProperties, String gmlProperties, String gmlClassesProperties) 
			throws InitializationException {
		initFactories(factoryProperties);
		initSimulator(simProperties);
		initOverlay(overlayProperties);
		initBehaviour(behProperties);
		initDHT(dhtProperties);
		initGraphicalGML(gmlProperties);
		initGMLClasses(gmlClassesProperties);
	}
	
	/**
	 * Initializa all attributes for factories.
	 * @param filename File which contains all properties for the factories.
	 * @throws InitializationException if occur some error during initialization.
	 */
	public static void initFactories(String filename) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		String factoriesFault = "";
		java.util.Properties prop = new java.util.Properties(); 
		try {
			fis = new FileInputStream(filename);
			prop.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+filename,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+filename,e);
		}
		//if all OK
		
		//read defaultNetwork
		try {
			networkClass = Class.forName(prop.getProperty(DEFAULT_NETWORK));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NETWORK +"] is not a valid class name.",e);
		} 
				
		//read defaultApplication
		try {
			appClass = Class.forName(prop.getProperty(DEFAULT_APPLICATION));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_APPLICATION +"] is not a valid class name.",e);
		} 
		
		//read defaultIdClass
		try {
			idClass = Class.forName(prop.getProperty(DEFAULT_ID_CLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_ID_CLASS +"] is not a valid class name.",e);
		} 
		
		//read defaultIdFactory
		try {
			idFactory = Class.forName(prop.getProperty(DEFAULT_IDFACTORY));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_IDFACTORY +"] is not a valid class name.",e);
		}

		//read defaultBehavioursPool
		if (prop.containsKey(DEFAULT_BEHCLASS)) {
			try {
				behClass = Class.forName(prop.getProperty(DEFAULT_BEHCLASS));
			} catch(ClassNotFoundException e) {
				throw new InitializationException("Property ["+DEFAULT_BEHCLASS +"] is not a valid class name.",e);
			}
		}
		
		//read defaultBehavioursPoolFactory
		if (prop.containsKey(DEFAULT_BEHFACTORY)) {
			try {
				behFactoryClass = Class.forName(prop.getProperty(DEFAULT_BEHFACTORY));
			} catch(ClassNotFoundException e) { 
				throw new InitializationException("Property ["+DEFAULT_BEHFACTORY +"] is not a valid class name.",e);
			}
		}
		
		//read defaultNodeClass
		try {
			nodeClass = Class.forName(prop.getProperty(DEFAULT_NODECLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NODECLASS +"] is not a valid class name.",e);
		}
		
		//read defaultNodeFactory
		try {
			nodeFactClass = Class.forName(prop.getProperty(DEFAULT_NODE_FACTORY));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property "+DEFAULT_NODE_FACTORY +" is not a valid class name ["+ 
					prop.getProperty(DEFAULT_NODE_FACTORY) +"].",e);
		} 
		
		//read DEFAULT_NODEHANDLECLASS
		try {
			nodeHandleClass = Class.forName(prop.getProperty(DEFAULT_NODEHANDLECLASS));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_NODECLASS +"] is not a valid class name.",e);
		}
		
		//read defaultTopology of network
		topology = prop.getProperty(DEFAULT_TOPOLOGY);
		if (!Topology.isValid(topology))
			throw new InitializationException("The topology ["+topology+"] is not a valid topology.");
		
		//read the defaultSize of network.
		networkSize = Integer.parseInt(prop.getProperty(DEFAULT_NETWORK_SIZE));
		if (networkSize < 0) {
			throw new InitializationException("The network size ["+networkSize+"] are invalid.");
		}
		
		//read defaultBitskey
		bitsKey = Integer.parseInt(prop.getProperty(DEFAULT_BITS_KEY));
		if (bitsKey > 160 || bitsKey%32!=0) {
			throw new InitializationException("The property ["+DEFAULT_BITS_KEY+"] has an invalid value. "
				+ "The only correct value is 160 or any 32*X in 32..160 range.");
		}
		
		//read defaultSerializedFile
		serializedFile = prop.getProperty(DEFAULT_SERIALIZED_FILE);
		
		//read defaultRouteMessage
		try {
			routeMessage = Class.forName(prop.getProperty(DEFAULT_ROUTE_MESSAGE));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_ROUTE_MESSAGE +"] is not a valid class name.",e);
		}

		//read DEFAULT_END_POINT
		try {
			endPoint = Class.forName(prop.getProperty(DEFAULT_END_POINT));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_END_POINT +"] is not a valid class name.",e);
		}
		
	}	
	
	/**
	 * Load all properties for the simulator.
	 * @param filename File that contains all properties for simulator.
	 * @throws InitializationException if occur some error during initialization.
	 */
	public static void initSimulator(String filename) throws InitializationException {
		//Load properties
		FileInputStream fis = null;
		java.util.Properties prop = new java.util.Properties();
		try {
			fis = new FileInputStream(filename);
			prop.load(fis);
		} catch (FileNotFoundException e) {
			throw new InitializationException("Cannot read the file "+filename,e);
		} catch (IOException e) {
			throw new InitializationException("Cannot load the file "+filename,e);
		}
		
		//if all OK

		//read the defaultEventFile
		eventFile = prop.getProperty(DEFAULT_EVENT_FILE);

		//read the defaultStabilizationSteps
		simulationSteps = Integer.parseInt(prop.getProperty(DEFAULT_SIMULATION_STEPS));
		if (simulationSteps < 0) 
			throw new InitializationException("Property ["+DEFAULT_SIMULATION_STEPS +"] is not a valid number of steps.");
		
		//read the defaultLogLevel number
		logLevel = Integer.parseInt(prop.getProperty(DEFAULT_LOG_LEVEL));
		if (logLevel < 0 || logLevel > 3) 
			throw new InitializationException("Property ["+DEFAULT_LOG_LEVEL +"] is not a valid log level.");
		
		//read the defualtOutputFile 
		outputFile = prop.getProperty(DEFAULT_OUTPUT_FILE);
		
		//read the defaultReplaceOutputFile
		replaceOutputFile = Boolean.getBoolean(prop.getProperty(DEFAULT_REPLACE_OUTPUT_FILE));
		
		//read DEFAULT_TIMER
		try {
			timerClass = Class.forName(prop.getProperty(DEFAULT_TIMER));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_TIMER +"] is not a valid class name.",e);
		}
		
		//read DEFAULT_OVERLAY_PROPERTIES
		try {
			overlayPropertiesClass = Class.forName(prop.getProperty(DEFAULT_OVERLAY_PROPERTIES));
		} catch (ClassNotFoundException e) {
			throw new InitializationException("Property ["+DEFAULT_OVERLAY_PROPERTIES +"] is not a valid class name.",e);
		}
		
		//read DEFAULT_BEHAVIOUR_PROPERTIES
		if (prop.containsKey(DEFAULT_BEHAVIOUR_PROPERTIES)) {
			try {
				behPropertiesClass = Class.forName(prop.getProperty(DEFAULT_BEHAVIOUR_PROPERTIES));
			} catch (ClassNotFoundException e) {
				throw new InitializationException("Property ["+DEFAULT_BEHAVIOUR_PROPERTIES +"] is not a valid class name.",e);
			}
		}
		
		//read DEFAULT_DHT_PROPERTIES
		if (prop.containsKey(DEFAULT_DHT_PROPERTIES)) {
			try {
				dhtPropertiesClass = Class.forName(prop.getProperty(DEFAULT_DHT_PROPERTIES));
			} catch (ClassNotFoundException e) {
				throw new InitializationException("Property ["+DEFAULT_DHT_PROPERTIES +"] is not a valid class name.",e);
			}
		}
		
		// read DEFAULT_GML_PROPERTIES
		if (prop.containsKey(DEFAULT_GML_PROPERTIES)) {
			try {
				gmlPropsClass = Class.forName(prop.getProperty(DEFAULT_GML_PROPERTIES));
			} catch (ClassNotFoundException e) {
				throw new InitializationException("Property ["+DEFAULT_GML_PROPERTIES +"] is not a valid class name.",e);
			}
		}
		
		//read the defaultQueueSize
		queueSize = Integer.parseInt(prop.getProperty(DEFAULT_QUEUE_SIZE));
		if (queueSize < 0) 
			throw new InitializationException("Property ["+DEFAULT_QUEUE_SIZE +"] is not a valid number of queue size.");
		
		//read the defaultMsgProcess
		msgProcess = Integer.parseInt(prop.getProperty(DEFAULT_MSG_PROCESS));
		if (msgProcess < 0) 
			throw new InitializationException("Property ["+DEFAULT_MSG_PROCESS +"] is not a valid number of message process.");

	}
	
	/**
	 * Initialize the overlay properties.
	 * @param overlayProperties Filename with all the configuration properties for
	 * the concrete overlay.
	 */
	private static void initOverlay(String overlayProperties) throws InitializationException {
		try {
			overlayProps = (OverlayProperties)overlayPropertiesClass.newInstance();
			overlayProps.init(overlayProperties);
		} catch (InitializationException e) {
			throw e;
		} catch (Exception e) {
			throw new InitializationException("Cannot build a new instance with the overlay properties.",e);
		} 
	}
	/**
	 * Initialize the behaviours properties.
	 * @param behaviuourProperties Filename with all the configuration properties for
	 * the concrete behaviour.
	 */
	private static void initBehaviour(String behProperties) throws InitializationException {
		try {
			behProps = (BehaviourProperties) behPropertiesClass.newInstance();
			behProps.init(behProperties);
		} catch (InitializationException e) {
			throw e;
		} catch (Exception e) {
			throw new InitializationException("Cannot build a new instance with the behaviours properties.",e);
		} 
	}
	/**
	 * Initialize the DHT properties.
	 * @param dhtProperties Filename with all the configuration properties for
	 * the concrete dht.
	 */
	private static void initDHT(String dhtProperties) throws InitializationException {
		try {
			dhtProps = (DHTProperties) dhtPropertiesClass.newInstance();
			dhtProps.init(dhtProperties);
		} catch (InitializationException e) {
			throw e;
		} catch (Exception e) {
			throw new InitializationException("Cannot build a new instance with the dht properties.",e);
		} 
	}
	
	/**
	 * Initialize all graphical GML properties.
	 * @param gmlProperties Filename with all the configuration properties for
	 * the graphical elements in GML graph.
	 */
	private static void initGraphicalGML(String gmlProperties) throws InitializationException {
		try {
			gmlProps = (PropertiesInitializer) gmlPropsClass.newInstance();
			gmlProps.init(gmlProperties);
		} catch (InitializationException e) {
			throw e;
		} catch (Exception e) {
			throw new InitializationException("Cannot build a new instance with the GML graphical properties.",e);
		} 
	}
	
	/**
	 * Initialize all GML classes appeared in <b>gmlProperties</b>.
	 * @param gmlProperties Filename with all the classes for
	 * the elements in GML graph.
	 */
	private static void initGMLClasses(String gmlProperties) throws InitializationException {
		gmlClassesProperties = gmlProperties; 
	}

}
